using System.Collections;
using System.Collections.Generic;
using System.Linq;
using Upsilon.BaseTypes.Number;
using Upsilon.BaseTypes.ScriptTypeInterfaces;
using Upsilon.BoundTypes;
using Upsilon.Evaluator;
using Upsilon.Text;

namespace Upsilon.BaseTypes.UserData
{
    internal class DictionaryUserData : ScriptType, IUserData, ILengthType, IIterable
    {
        public IDictionary Dictionary { get; }
        private string BoundTypeName { get; }

        public DictionaryUserData(IDictionary dictionary)
        {
            Dictionary = dictionary;
            BoundTypeName = BoundTypeHandler.GetTypeName(dictionary.GetType());
        }

        public ScriptType Get(Diagnostics diagnostics, TextSpan span, ScriptType index, EvaluationScope scope)
        {
            var key = index.ToCSharpObject();
            if (Dictionary.Contains(key))
            {
                return Dictionary[key].ToScriptType();
            }
            //TODO: log error
            return new ScriptNull();
        }

        public void Set(Diagnostics diagnostics, TextSpan span, ScriptType index, ScriptType value)
        {
            var key = index.ToCSharpObject();
            if (Dictionary.Contains(key))
            {
                Dictionary[key] = value.ToCSharpObject();
            }
            else
            {
                Dictionary.Add(key, value.ToCSharpObject());
            }
        }

        public override TypeContainer Type => new TypeContainer(BoundTypeName);
        public override object ToCSharpObject()
        {
            return Dictionary;
        }

        public override System.Type GetCSharpType()
        {
            return Dictionary.GetType();
        }

        public ScriptNumberLong Length()
        {
            return new ScriptNumberLong(Dictionary.Count);
        }

        protected bool Equals(DictionaryUserData other)
        {
            return Dictionary.Equals(other.Dictionary);
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            if (ReferenceEquals(this, obj)) return true;
            if (obj.GetType() == Dictionary.GetType())
            {
                return Dictionary.Equals(obj);
            }
            if (obj.GetType() != this.GetType()) return false;
            return Equals((DictionaryUserData) obj);
        }

        public ScriptType GetValueFromIndex(ScriptType index)
        {
            return Dictionary[index.ToCSharpObject()].ToScriptType();
        }

        public IEnumerator<ScriptType> GetScriptEnumerator()
        {
            return (from object key in Dictionary.Keys
                select key.ToScriptType()).GetEnumerator();
        }
    }
}