using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Reflection;
using Upsilon.BoundTypes;
using Upsilon.Exceptions;
using Upsilon.StandardLibraries;
using Upsilon.Text;

namespace Upsilon.BaseTypes.UserData
{
    public static class UserDataTypeHandler
    {
        private static readonly ConcurrentDictionary<System.Type, UserDataType> Types
            = new ConcurrentDictionary<System.Type, UserDataType>();

        public static void LoadType(System.Type t, string name)
        {
            var info = new UserDataType(t);
            Types.AddOrUpdate(t, info, (type, dataType) => dataType);
            UserDataBoundTypeDefinition boundType;
            if (t.IsEnum)
            {
                boundType = new UserDataBoundEnumDefinition(t, name);
            }
            else
            {
                boundType = UserDataBoundTypeDefinition.Create(t, name);
            }
            BoundTypeHandler.LoadUserDataTypeDefinition(boundType);
        }

        public static void LoadAssembly(Assembly assembly)
        {
            var types = assembly.GetTypes();
            foreach (var type in types)
            {
                var attr = (UpsilonUserDataAttribute)type.GetCustomAttribute(typeof(UpsilonUserDataAttribute));
                if (attr != null)
                {
                    var name = attr.Name.ToLowerInvariant();
                    LoadType(type, name);
                    if (type.IsEnum)
                    {
                        var def = Activator.CreateInstance(type);
                        StaticScope.RegisterStaticVariable(attr.Name, def);
                    }
                }
            }
        }

        public static void LoadType<T>()
        {
            var attr = (UpsilonUserDataAttribute)typeof(T).GetCustomAttribute(typeof(UpsilonUserDataAttribute));
            var name = typeof(T).Name;
            if (attr != null) name = attr.Name;
            LoadType(typeof(T), name);
        }

        internal static UserDataType GetTypeInfo(System.Type t)
        {
            if (t.IsGenericType)
                t = t.GetGenericTypeDefinition();
            if (Types.TryGetValue(t, out var result))
                return result;
            else
            {
                throw new Exception($"Can't use type '{t.FullName}' as script type, it's not registered for use.");
            }
        }
    }
}