using System;
using System.Collections.Generic;
using System.Collections.Immutable;
using Upsilon.BaseTypes;
using Upsilon.Text;
using Type = Upsilon.BaseTypes.Type;

namespace Upsilon.Binder
{
    public class BoundFunctionExpression : BoundExpression, IBoundNodeScopeOwner
    {
        public ImmutableArray<BoundVariableSymbol> Parameters { get; }
        public BoundBlockStatement Block { get; set; }

        public BoundFunctionExpression(ImmutableArray<BoundVariableSymbol> parameters, BoundBlockStatement block,
            TextSpan span, BoundScope scope, Type returnType) : base(span)
        {
            Parameters = parameters;
            Block = block;
            Scope = scope;
            ReturnType = returnType;
        }

        public override BoundKind Kind => BoundKind.BoundFunctionExpression;

        public override IEnumerable<BoundNode> GetChildren()
        {
            foreach (var parameter in Parameters)
            {
                yield return parameter;
            }
            yield return Block;
        }

        public override TypeContainer Type => BaseTypes.Type.Function;
        public BoundScope Scope { get; set; }
        public Type ReturnType { get; }
    }
}