using System.Collections.Generic;
using Upsilon.Text;

namespace Upsilon.Binder
{
    public abstract class BoundNode
    {
        protected BoundNode(TextSpan span)
        {
            Span = span;
        }

        public abstract BoundKind Kind { get; }
        public TextSpan Span { get; }

        public virtual IEnumerable<BoundNode> GetNodeAtPosition(int linePosition, int characterPosition)
        {
            var e = GetChildren().GetEnumerator();
            while (e.MoveNext())
            {
                var current = e.Current;
                if (current == null)
                    continue;
                if (current.Span.Contains(linePosition, characterPosition))
                {
                    var children = current.GetNodeAtPosition(linePosition, characterPosition);
                    foreach (var child in children)
                    {
                        yield return child;
                    }
                }
            }
            e.Dispose();

            if (Span.Contains(linePosition, characterPosition))
                yield return this;
        }
        public abstract IEnumerable<BoundNode> GetChildren();
    }
}