using System.Collections.Generic;
using Upsilon.Text;

namespace Upsilon.Binder
{
    public class BoundIfStatement : BoundStatement
    {
        public BoundIfStatement(BoundExpressionStatement condition, BoundBlockStatement block, TextSpan span): base(span)
        {
            Condition = condition;
            Block     = block;
        }

        public BoundIfStatement(BoundExpressionStatement condition, BoundBlockStatement block, BoundElseStatement elseStatement, TextSpan span) : base(span)
        {
            Condition     = condition;
            Block         = block;
            ElseStatement = elseStatement;
        }
        public BoundIfStatement(BoundExpressionStatement condition, BoundBlockStatement block, BoundIfStatement nextElseIf, TextSpan span) : base(span)
        {
            Condition     = condition;
            Block         = block;
            NextElseIf = nextElseIf;
        }

        public override BoundKind Kind => BoundKind.BoundIfStatement;
        public override IEnumerable<BoundNode> GetChildren()
        {
            yield return Condition;
            yield return Block;
            if (NextElseIf != null) yield return NextElseIf;
            if (ElseStatement != null) yield return ElseStatement;
        }

        public BoundExpressionStatement Condition { get; }
        public BoundBlockStatement Block { get; }
        public BoundIfStatement NextElseIf { get; }
        public BoundElseStatement ElseStatement { get; }
    }

    public class BoundElseStatement : BoundStatement
    {
        public BoundBlockStatement Block { get; }

        public BoundElseStatement(BoundBlockStatement block, TextSpan span) : base(span)
        {
            Block = block;
        }

        public override BoundKind Kind => BoundKind.BoundElseStatement;

        public override IEnumerable<BoundNode> GetChildren()
        {
            yield return Block;
        }
    }
}