using System;
using System.Linq;
using Upsilon.Parser;
using Type = Upsilon.BaseTypes.Type;

namespace Upsilon.Binder
{
    public class BoundUnaryOperator
    {
        public enum OperatorKind
        {
            Identity,
            Negation,
            LogicalNegation,
            Length
        }

        private Type InType { get; }
        public Type OutType { get; }
        public OperatorKind Kind { get; }

        private BoundUnaryOperator(OperatorKind kind, Type inType, Type outType)
        {
            Kind = kind;
            InType = inType;
            OutType = outType;
        }

        private static readonly BoundUnaryOperator[] Operators= {
            new BoundUnaryOperator(OperatorKind.Identity, Type.Number, Type.Number),
            new BoundUnaryOperator(OperatorKind.Negation, Type.Number, Type.Number),
            new BoundUnaryOperator(OperatorKind.LogicalNegation, Type.Boolean, Type.Boolean),
            new BoundUnaryOperator(OperatorKind.Length, Type.String, Type.Number),
            new BoundUnaryOperator(OperatorKind.Length, Type.Table, Type.Number),
            new BoundUnaryOperator(OperatorKind.Length, Type.UserData, Type.Number),
        };

        public static BoundUnaryOperator Bind(SyntaxKind operatorToken, Type inType)
        {
            OperatorKind kind;
            switch (operatorToken)
            {
                case SyntaxKind.Plus:
                    kind = OperatorKind.Identity;
                    break;
                case SyntaxKind.Minus:
                    kind = OperatorKind.Negation;
                    break;
                case SyntaxKind.NotKeyword:
                    kind = OperatorKind.LogicalNegation;
                    break;
                case SyntaxKind.PoundSign:
                    kind = OperatorKind.Length;
                    break;
                default:
                    throw new Exception("Unknown unary operator token: " + operatorToken);
            }

            if (inType == Type.Unknown)
                return Operators.FirstOrDefault(op => op.Kind == kind);
            return Operators.FirstOrDefault(op => op.Kind == kind && op.InType == inType);
        }
    }
}