using System.Collections.Immutable;
using System.Linq;
using Upsilon.BaseTypes;

namespace Upsilon.Binder.VariableSymbols
{
    public class ScriptFunctionVariableSymbol : FunctionVariableSymbol
    {
        public ScriptFunctionVariableSymbol(string name, bool local, ImmutableArray<VariableSymbol> parameters, Type resultType)
            : base(name, local, resultType)
        {
            FunctionOption.Add(new ScriptFunctionVariableOption(resultType, parameters));
        }

        public override (bool IsValid, string Error, BoundExpression WrongParameter) ValidateParameters(ImmutableArray<BoundExpression> callingParameters)
        {
            foreach (var functionVariableSymbolOption in FunctionOption)
            {
                if (!(functionVariableSymbolOption is ScriptFunctionVariableOption option))
                    continue;
                if (option.Parameters.Length != callingParameters.Length)
                    continue;
                bool isValid = true;
                for (var i = 0; i < option.Parameters.Length; i++)
                {
                    var functionParameter = option.Parameters[i];
                    var callingParameter  = callingParameters[i];
                    if (functionParameter.TypeContainer != BaseTypes.Type.Unknown &&
                        callingParameter.Type           != BaseTypes.Type.Unknown &&
                        callingParameter.Type           != BaseTypes.Type.Nil)
                    {
                        if (callingParameter.Type != functionParameter.TypeContainer)
                        {
                            isValid = false;
                            break;
                        }
                    }
                }
                if (!isValid)
                    continue;
                return (true, null, null);
            }

            return (false, null, null);
        }
    }

    public class ScriptFunctionVariableOption : FunctionVariableSymbolOption
    {
        public ImmutableArray<VariableSymbol> Parameters { get; }
        public bool                           IsBound    { get; set; }

        public ScriptFunctionVariableOption(Type resultType, ImmutableArray<VariableSymbol> parameters) : base(resultType)
        {
            Parameters = parameters;
        }

        public override TypeContainer[] GetParameterTypes()
        {
            return Parameters.Select(x => x.TypeContainer).ToArray();
        }
    }
}