using System.Collections.Generic;
using System.Xml.Linq;

namespace Upsilon.Evaluator.Debugging
{
    public static class DebugSession
    {
        public static bool DebuggerAttached { get; set; }
        internal static bool Debugging { get; set; }
        private static Dictionary<string, DebugFile> Files { get; set; }

        public static void AttachDebugger()
        {
            DebuggerAttached = true;
        }

        public static DebugFile GetFileInfo(string file)
        {
            return Files.TryGetValue(file, out var info) ? info : null;
        }

        public static void SetBreakpoint(string file, int line)
        {
            if (!Files.TryGetValue(file, out var fileObj))
            {
                fileObj = new DebugFile();
                Files.Add(file, fileObj);
            }
            fileObj.Breakpoints.Add(new Breakpoint()
            {
                Line = line
            });
        }

        public static void RemoveBreakpoint(string file, int line)
        {
            if (Files.TryGetValue(file, out var fileObj))
            {
                fileObj = new DebugFile();
                fileObj.Breakpoints.RemoveAll(x => x.Line == line);
                if (fileObj.Breakpoints.Count <= 0)
                    Files.Remove(file);
            }
        }

        internal static void TriggerBreakpoint(EvaluationScope scope)
        {
            OnBreakpoint?.Invoke(new BreakpointInfo(scope));
        }

        public delegate void BreakpointEvent(BreakpointInfo info);
        public static event BreakpointEvent OnBreakpoint;
    }
}