using System.Collections.Generic;
using System.Collections.Immutable;
using Upsilon.Text;

namespace Upsilon.Parser
{
    public sealed class TableExpressionSyntax : ExpressionSyntax
    {
        public TableExpressionSyntax(SyntaxToken openBrace, ImmutableArray<SyntaxNode> expressions, SyntaxToken closeBrace)
        {
            Expressions = expressions;
            OpenBrace = openBrace;
            CloseBrace = closeBrace;
            Span = TextSpan.Between(OpenBrace.Span, CloseBrace.Span);
        }

        public override SyntaxKind Kind => SyntaxKind.TableExpression;

        public SyntaxToken OpenBrace { get; }
        public ImmutableArray<SyntaxNode> Expressions { get; }
        public SyntaxToken CloseBrace { get; }

        public override IEnumerable<SyntaxNode> ChildNodes()
        {
            yield return OpenBrace;
            foreach (var e in Expressions)
                yield return e;
            yield return CloseBrace;
        }
    }
}