using System.Collections.Generic;
using System.Collections.Immutable;
using Upsilon.Text;

namespace Upsilon.Parser
{
    public class GenericForStatementSyntax : StatementSyntax
    {
        public SyntaxToken ForKeyword { get; }
        public ImmutableArray<IdentifierToken> Variables { get; }
        public SyntaxToken InKeyword { get; }
        public ExpressionSyntax EnumerableExpression { get; }
        public SyntaxToken DoKeyword { get; }
        public BlockStatementSyntax Block { get; }
        public SyntaxToken EndKeyword { get; }

        public GenericForStatementSyntax(SyntaxToken forKeyword, ImmutableArray<IdentifierToken> variables, SyntaxToken inKeyword,
            ExpressionSyntax enumerableExpression, SyntaxToken doKeyword, BlockStatementSyntax block, SyntaxToken endKeyword)
        {
            ForKeyword = forKeyword;
            Variables = variables;
            InKeyword = inKeyword;
            EnumerableExpression = enumerableExpression;
            DoKeyword = doKeyword;
            Block = block;
            EndKeyword = endKeyword;
            Span = TextSpan.Between(forKeyword.Span, endKeyword.Span);
        }

        public override SyntaxKind Kind => SyntaxKind.GenericForStatement;
        public override IEnumerable<SyntaxNode> ChildNodes()
        {
            yield return ForKeyword;
            foreach (var identifierToken in Variables)
                yield return identifierToken;
            yield return InKeyword;
            yield return EnumerableExpression;
            yield return DoKeyword;
            yield return Block;
            yield return EndKeyword;
        }
    }
}