using System;
using System.Collections.Generic;
using System.Collections.Immutable;
using System.Linq;
using Upsilon.Text;

namespace Upsilon.Parser
{
    public class IfStatementSyntax : StatementSyntax
    {
        public IfStatementSyntax(SyntaxToken ifToken, ExpressionStatementSyntax condition, SyntaxToken thenToken,
            BlockStatementSyntax block, SyntaxToken endToken)
        {
            IfToken = ifToken;
            Condition = condition;
            ThenToken = thenToken;
            Block = block;
            EndToken = endToken;
            var end = Block.Span;
            if (EndToken != null) end = EndToken.Span;
            Span = TextSpan.Between(ifToken.Span, end);
        }

        public IfStatementSyntax(SyntaxToken ifToken, ExpressionStatementSyntax condition, SyntaxToken thenToken,
            BlockStatementSyntax block, ElseStatementSyntax elseStatement)
        {
            IfToken       = ifToken;
            Condition     = condition;
            ThenToken     = thenToken;
            Block         = block;
            ElseStatement = elseStatement;
            Span = TextSpan.Between(ifToken.Span, elseStatement.Span);
        }

        public IfStatementSyntax(SyntaxToken ifToken, ExpressionStatementSyntax condition, SyntaxToken thenToken,
            BlockStatementSyntax block, ElseIfStatementSyntax nextElseIfStatement)
        {
            IfToken       = ifToken;
            Condition     = condition;
            ThenToken     = thenToken;
            Block         = block;
            NextElseIfStatement = nextElseIfStatement;
            Span = TextSpan.Between(ifToken.Span, nextElseIfStatement.Span);
        }


        public SyntaxToken IfToken { get; }
        public ExpressionStatementSyntax Condition { get; }
        public SyntaxToken ThenToken { get; }
        public BlockStatementSyntax Block { get; }
        public ElseIfStatementSyntax NextElseIfStatement { get; protected set; }
        public ElseStatementSyntax ElseStatement { get; protected set; }
        public SyntaxToken EndToken { get; }

        public override SyntaxKind Kind => SyntaxKind.IfStatement;
        public override IEnumerable<SyntaxNode> ChildNodes()
        {
            yield return IfToken;
            yield return Condition;
            yield return ThenToken;
            yield return Block;
            if (NextElseIfStatement != null)
                yield return NextElseIfStatement;
            if (ElseStatement != null)
                yield return ElseStatement;
            if (EndToken != null)
                yield return EndToken;
        }
    }

    public sealed class ElseIfStatementSyntax : IfStatementSyntax
    {
        public SyntaxToken ElseIfToken => IfToken;

        public override SyntaxKind Kind => SyntaxKind.ElseIfStatement;
        public override IEnumerable<SyntaxNode> ChildNodes()
        {
            yield return ElseIfToken;
            yield return Condition;
            yield return ThenToken;
            yield return Block;
            Span = TextSpan.Between(ElseIfToken.Span, Block.Span);
        }

        public ElseIfStatementSyntax(IfStatementSyntax s)
        :base(s.IfToken, s.Condition, s.ThenToken, s.Block, s.EndToken)
        {
            NextElseIfStatement = s.NextElseIfStatement;
            ElseStatement = s.ElseStatement;
        }

        public ElseIfStatementSyntax(SyntaxToken ifToken, ExpressionStatementSyntax condition, SyntaxToken thenToken,
            BlockStatementSyntax block, SyntaxToken endToken) : base(ifToken, condition, thenToken, block, endToken)
        {
        }

        public ElseIfStatementSyntax(SyntaxToken ifToken, ExpressionStatementSyntax condition, SyntaxToken thenToken,
            BlockStatementSyntax block, ElseStatementSyntax elseStatement) : base(ifToken, condition, thenToken, block,
            elseStatement)
        {
        }

        public ElseIfStatementSyntax(SyntaxToken ifToken, ExpressionStatementSyntax condition, SyntaxToken thenToken,
            BlockStatementSyntax block, ElseIfStatementSyntax nextElseIfStatement) : base(ifToken, condition, thenToken,
            block, nextElseIfStatement)
        {
        }
    }


    public sealed class ElseStatementSyntax : StatementSyntax
    {
        public SyntaxToken ElseToken { get; }
        public BlockStatementSyntax Block { get; }
        public SyntaxToken EndToken { get; }

        public ElseStatementSyntax(SyntaxToken elseToken, BlockStatementSyntax block, SyntaxToken endToken)
        {
            ElseToken = elseToken;
            Block = block;
            EndToken = endToken;
            Span = TextSpan.Between(elseToken.Span, endToken.Span);
        }

        public override SyntaxKind Kind => SyntaxKind.ElseStatement;
        public override IEnumerable<SyntaxNode> ChildNodes()
        {
            yield return ElseToken;
            yield return Block;
            yield return EndToken;
        }
    }
}