using System.Collections.Generic;
using System.Collections.Immutable;
using System.Linq;
using Upsilon.Text;

namespace Upsilon.Parser
{
    public class MultiAssignmentStatementSyntax : StatementSyntax
    {
        public SyntaxToken LocalKeyword { get; }
        public ImmutableArray<IdentifierToken> Identifiers { get; }
        public SyntaxToken AssignmentToken { get; }
        public ExpressionSyntax Expression { get; }

        public MultiAssignmentStatementSyntax(SyntaxToken localKeyword, IEnumerable<IdentifierToken> identifiers,
            SyntaxToken assignmentToken, ExpressionSyntax expression)
        {
            LocalKeyword = localKeyword;
            Identifiers = identifiers.ToImmutableArray();
            AssignmentToken = assignmentToken;
            Expression = expression;
            var start = Identifiers.First().Span;
            if (localKeyword != null) start = localKeyword.Span;
            Span = TextSpan.Between(start, expression.Span);
        }

        public override SyntaxKind Kind => SyntaxKind.MultiAssignmentStatement;
        public override IEnumerable<SyntaxNode> ChildNodes()
        {
            yield return LocalKeyword;
            foreach (var expressionSyntax in Identifiers)
                yield return expressionSyntax;
            yield return AssignmentToken;
            yield return Expression;
        }
    }
}