using System.Collections.Generic;
using Upsilon.Text;

namespace Upsilon.Parser
{
    public class NumericForStatementSyntax : StatementSyntax
    {
        public SyntaxToken ForToken { get; }
        public IdentifierToken Identifier { get; }
        public SyntaxToken EqualsToken { get; }
        public ExpressionSyntax StartExpression { get; }
        public SyntaxToken Comma1 { get; }
        public ExpressionSyntax StopExpression { get; }
        public SyntaxToken Comma2 { get; }
        public ExpressionSyntax StepExpression { get; }
        public SyntaxToken DoToken { get; }
        public StatementSyntax Block { get; }
        public SyntaxToken EndToken { get; }

        public NumericForStatementSyntax(SyntaxToken forToken, IdentifierToken identifier, SyntaxToken equalsToken,
            ExpressionSyntax startExpression, SyntaxToken comma1, ExpressionSyntax stopExpression, SyntaxToken comma2,
            ExpressionSyntax stepExpression, SyntaxToken doToken, StatementSyntax block, SyntaxToken endToken)
        {
            ForToken = forToken;
            Identifier = identifier;
            EqualsToken = equalsToken;
            StartExpression = startExpression;
            Comma1 = comma1;
            StopExpression = stopExpression;
            Comma2 = comma2;
            StepExpression = stepExpression;
            DoToken = doToken;
            Block = block;
            EndToken = endToken;
            Span = TextSpan.Between(forToken.Span, endToken.Span);
        }

        public override SyntaxKind Kind => SyntaxKind.NumericForStatement;
        public override IEnumerable<SyntaxNode> ChildNodes()
        {
            yield return ForToken;
            yield return Identifier;
            yield return EqualsToken;
            yield return StartExpression;
            yield return Comma1;
            yield return StopExpression;
            if (Comma2 != null)
                yield return Comma2;
            if (StepExpression != null)
                yield return StepExpression;
            yield return DoToken;
            yield return Block;
        }
    }
}