namespace Upsilon.Parser
{
    public static class SyntaxKindPrecedence
    {
        public enum Precedence
        {
            None = 0,
            LogicalOr,
            LogicalAnd,
            Equality,
            Additive,
            Multiplicative,
            Unary,
        }

        public static Precedence UnaryOperatorPrecedence(this SyntaxKind kind)
        {
            switch (kind)
            {
                case SyntaxKind.Plus:
                case SyntaxKind.Minus:
                case SyntaxKind.NotKeyword:
                case SyntaxKind.PoundSign:
                    return Precedence.Unary;
                default:
                    return Precedence.None;
            }
        }

        public static Precedence BinaryOperatorPrecedence(this SyntaxKind kind)
        {
            switch (kind)
            {
                // equality operators
                case SyntaxKind.EqualsEquals:
                case SyntaxKind.TildeEquals:

                case SyntaxKind.Greater:
                case SyntaxKind.GreaterEquals:
                case SyntaxKind.Less:
                case SyntaxKind.LessEquals:
                    return Precedence.Equality;

                // logical operators
                case SyntaxKind.AndKeyword:
                    return Precedence.LogicalAnd;
                case SyntaxKind.OrKeyword:
                    return Precedence.LogicalOr;

                // math operators
                case SyntaxKind.Plus:
                case SyntaxKind.Minus:
                    return Precedence.Additive;

                case SyntaxKind.Star:
                case SyntaxKind.Slash:
                case SyntaxKind.PercentSign:
                case SyntaxKind.RoofSign:
                    return Precedence.Multiplicative;



                default:
                    return Precedence.None;
            }

        }
    }
}